@@ -9,7 +9,7 @@ License: Public Domain
 """
 
 import hxl_proxy
-from hxl.io import HXLIOException
+from hxl.input import HXLIOException
 
 from hxl_proxy import admin, app, auth, cache, caching, dao, exceptions, filters, pcodes, preview, recipes, util, validate
 
@@ -106,7 +106,7 @@ def handle_source_authorization_exception(e):
     return flask.redirect(util.data_url_for('data_save', recipe=recipe, extras=extras), 302)
 
 # register the source authorisation handler
-app.register_error_handler(hxl.io.HXLAuthorizationException, handle_source_authorization_exception)
+app.register_error_handler(hxl.input.HXLAuthorizationException, handle_source_authorization_exception)
 
 
 def handle_password_required_exception(e):
@@ -252,29 +252,10 @@ def data_tagger(recipe_id=None):
     if header_row is not None:
         header_row = int(header_row)
         
-    try:
-        sheet_index = int(recipe.args.get('sheet', 0))
-    except:
-        logger.info("Assuming sheet 0, since none specified")
-        sheet_index = 0
-
-    selector = recipe.args.get('selector', None)
-
     # Set up a 25-row raw-data preview, using make_input from libhxl-python
     preview = []
     i = 0
-    http_headers = {
-        'User-Agent': 'hxl-proxy/tagger'
-    }
-    if 'authorization_token' in recipe.args: # private dataset
-        http_headers['Authorization'] = recipe.args['authorization_token']
-    for row in hxl.io.make_input(
-            recipe.url,
-            sheet_index=sheet_index,
-            selector=selector,
-            verify_ssl=util.check_verify_ssl(recipe.args),
-            http_headers=http_headers
-    ):
+    for row in hxl.input.make_input(recipe.url, util.make_input_options(recipe.args)):
         # Stop if we get to 25 rows
         if i >= 25:
             break
@@ -329,7 +310,7 @@ def data_edit(recipe_id=None):
     except exceptions.RedirectException as e1:
         # always pass through a redirect exception
         raise e1
-    except hxl.io.HXLAuthorizationException as e2:
+    except hxl.input.HXLAuthorizationException as e2:
         # always pass through an authorization exception
         raise e2
     except Exception as e3:
@@ -413,11 +394,7 @@ def data_validate(recipe_id=None, format='html'):
     # Set up the HXL validation schema
     schema_source = None
     if recipe.schema_url:
-        schema_source = hxl.data(
-            recipe.schema_url,
-            verify_ssl=util.check_verify_ssl(recipe.args),
-            http_headers={'User-Agent': 'hxl-proxy/validation'}
-        )
+        schema_source = hxl.data(recipe.schema_url, util.make_input_options(recipe.args))
         logger.info("Using HXL validation schema at %s", recipe.schema_url)
     else:
         logger.info("No HXL validation schema specified; using default schema")
@@ -754,7 +731,7 @@ def do_data_save():
 # has tests
 @app.route("/actions/validate", methods=['POST'])
 def do_data_validate():
-    """ Flask controler: validate an uploaded file against an uploaded HXL schema
+    """ Flask controller: validate an uploaded file against an uploaded HXL schema
     This controller was created for HDX Data Check, which is the only known user.
     The controller returns a JSON validation report from libhxl-python.
 
@@ -812,7 +789,7 @@ def do_data_validate():
     report = validate.run_validation(
         url, content, content_hash, sheet_index, selector,
         schema_url, schema_content, schema_content_hash, schema_sheet_index,
-        include_dataset
+        include_dataset, flask.request.form
     )
 
     # return a JSON version of the validation report as an HTTP response
@@ -870,7 +847,7 @@ def do_json_recipe():
     flask.g.output_format = 'format'
 
     # Create a HXL filter chain by parsing the JSON recipe
-    source = hxl.io.from_spec(json_recipe)
+    source = hxl.input.from_spec(json_recipe)
 
     # Create a JSON or CSV response object, as requested
     if format == 'json':
@@ -1104,7 +1081,6 @@ def from_spec(format="json"):
         flask.g.output_format = format
 
     # other args
-    verify_ssl = util.check_verify_ssl(flask.request.args)
     http_headers = {
         'User-Agent': 'hxl-proxy/download'
     }
@@ -1121,9 +1097,9 @@ def from_spec(format="json"):
             'api-from-spec.html',
             spec_json=spec_json,
             spec_url=spec_url,
-            verify_ssl=verify_ssl,
+            verify_ssl=util.check_verify_ssl(flask.request.args),
             filename=filename,
-            force=force
+            force=force,
         )
     elif spec_url and spec_json:
         raise ValueError("Must specify only one of &spec-url or &spec-json")
@@ -1137,7 +1113,7 @@ def from_spec(format="json"):
         raise ValueError("Either &spec-url or &spec-json required")
 
     # process the JSON spec
-    source = hxl.io.from_spec(spec)
+    source = hxl.input.from_spec(spec)
 
     # produce appropriate output
     if format == "json":
@@ -1215,11 +1191,7 @@ def hxl_test(format='html'):
     if url:
         try:
             # we grab the columns to force lazy parsing
-            hxl.data(
-                url,
-                verify_ssl=util.check_verify_ssl(flask.request.args),
-                http_headers={'User-Agent': 'hxl-proxy/test'}
-            ).columns
+            hxl.data(url, util.make_input_options(flask.request.args)).columns
             # if we get to here, it's OK
             result['status'] = True
             result['message'] = 'Dataset has HXL hashtags'
@@ -1227,7 +1199,7 @@ def hxl_test(format='html'):
             # can't open resource to check it
             result['message'] = 'Cannot load dataset'
             record_exception(e1)
-        except hxl.io.HXLTagsNotFoundException as e2:
+        except hxl.input.HXLTagsNotFoundException as e2:
             # not hashtagged
             result['message'] = 'Dataset does not have HXL hashtags'
             record_exception(e2)
@@ -1317,38 +1289,24 @@ def data_preview (format="json"):
     # params
     url = flask.request.args.get('url')
 
-    sheet = flask.request.args.get('sheet')
-    if sheet is not None:
-        sheet = int(sheet)
-
-    rows = flask.request.args.get('rows')
-    if rows is not None:
-        rows = int(rows)
-
-    force = flask.request.args.get('force')
+    rows = flask.request.args.get('rows', 0)
+    rows = int(rows)
 
     filename = flask.request.args.get('filename')
 
     if format == "html":
-        return flask.render_template('api-data-preview.html', url=url, sheet=sheet, rows=rows, filename=filename, force=force)
+        return flask.render_template('api-data-preview.html', url=url, args=flask.request.args)
 
     # if there's no URL, then show an interactive form
     if not url:
         return flask.redirect('/api/data-preview.html', 302)
 
-    # fix up params
-    # if not sheet:
-    #     sheet = -1
-    #
-    if not rows:
-        rows = -1
-
     # make input
     if util.skip_cache_p():
-        input = hxl.io.make_input(url, sheet_index=sheet)
+        input = hxl.input.make_input(url, util.make_input_options(flask.request.args))
     else:
         with caching.input():
-            input = hxl.io.make_input(url, sheet_index=sheet)
+            input = hxl.input.make_input(url, util.make_input_options(flask.request.args))
 
     # Generate result
     if format == 'json':
@@ -1418,14 +1376,18 @@ def data_preview_sheets(format="json"):
 
     # make input
     _output = []
+
+    args = dict(flask.request.args)
+    
     try:
         for sheet in range(0, SHEET_MAX_NO):
+            args['sheet'] = sheet
             if util.skip_cache_p():
-                input = hxl.io.make_input(url, sheet_index=sheet)
+                input = hxl.input.make_input(url, util.make_input_options(args))
             else:
                 with caching.input():
-                    input = hxl.io.make_input(url, sheet_index=sheet)
-            if isinstance(input, hxl.io.CSVInput):
+                    input = hxl.input.make_input(url, util.make_input_options(args))
+            if isinstance(input, hxl.input.CSVInput):
                 _output.append("Default")
                 break
             else:
@@ -1494,7 +1456,7 @@ def make_hash():
     headers_only = flask.request.args.get('headers_only')
 
     # Open the HXL dataset
-    source = hxl.data(url)
+    source = hxl.data(url, util.make_input_options(flask.request.args))
 
     # Generate the report
     report = {
@@ -1513,6 +1475,33 @@ def make_hash():
     )
 
 
+# has tests
+@app.route('/api/source-info')
+def make_info():
+    """ Flask controller: get info for an Excel dataset
+    GET parameters:
+    url - the URL of the dataset to check
+    """
+    flask.g.output_format = 'json' # for error reporting
+
+    url = flask.request.args.get('url')
+    if not url:
+        raise ValueError("Parameter 'url' is required")
+
+    # Open the dataset (not necessarily hxlated)
+    try:
+        info = hxl.make_input(url, util.make_input_options(flask.request.args)).info()
+        return flask.Response(
+            json.dumps(info, indent=4),
+            mimetype="application/json"
+        )
+    except NotImplementedError:
+        return flask.Response(
+            { "error": "dataset format not supported" },
+            mimetype="application/json",
+            status=400
+        )
+
 
 ########################################################################
 # Controllers for removed features (display error messages)
